from {{ current_model.record_service_config_class|package_name }} import {{ current_model.record_service_config_class|base_name }}
from {{ current_model.record_service_class|package_name }} import {{ current_model.record_service_class|base_name }}
from {{ current_model.record_resource_config_class|package_name }} import {{ current_model.record_resource_config_class|base_name }}
from {{ current_model.record_resource_class|package_name }} import {{ current_model.record_resource_class|base_name }}


{{ current_model.config_resource_config_key }} = {{ current_model.record_resource_config_class|base_name }}
{{ current_model.config_resource_class_key }} = {{ current_model.record_resource_class|base_name }}
{{ current_model.config_service_config_key }} = {{ current_model.record_service_config_class|base_name }}
{{ current_model.config_service_class_key }} = {{ current_model.record_service_class|base_name }}

{{ current_model.invenio_config_extra_code }}