from invenio_records_resources.services import RecordService as InvenioRecordService



{% for b in current_model.record_service_bases %}
from {{ b|package_name }} import {{ b|base_name }}
{% endfor %}

class {{ current_model.record_service_class|base_name }}({% for b in current_model.record_service_bases %}{{ b|base_name }}{{ ", " if not loop.last else "" }}{% endfor %}):
    """{{ current_model.record_class|base_name }} service."""

{{ current_model.invenio_record_service_extra_code }}